var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// src/parser.ts
var PARSER_TYPES = {
  N: "null",
  i: "int",
  d: "float",
  b: "boolean",
  s: "string",
  a: "array-object",
  C: "serializable-class",
  O: "object"
};
var Parser = class {
  static {
    __name(this, "Parser");
  }
  index;
  contents;
  options;
  constructor(contents, index, options) {
    this.contents = contents;
    this.index = index;
    this.options = options;
  }
  error(message = "Syntax Error") {
    return new Error(`${message} at index ${this.index} while unserializing payload`);
  }
  advance(index) {
    this.index += index;
  }
  readAhead(index) {
    const contents = this.peekAhead(index);
    this.index += index;
    return contents;
  }
  readUntil(expected) {
    const index = this.contents.indexOf(expected, this.index);
    if (index === -1) {
      throw this.error(`Expected '${expected}'`);
    }
    return this.readAhead(index - this.index);
  }
  peekAhead(index) {
    return this.contents.toString(this.options.encoding, this.index, this.index + index);
  }
  seekExpected(contents) {
    const slice = this.readAhead(contents.length);
    if (slice !== contents) {
      this.index -= contents.length;
      throw this.error(`Expected '${contents}'`);
    }
  }
  getType() {
    const [type, ps] = this.readAhead(2);
    const parserType = PARSER_TYPES[type];
    if (!parserType) {
      throw this.error("Unknown type");
    }
    if (parserType === "null" ? ps !== ";" : ps !== ":") {
      throw this.error();
    }
    return parserType;
  }
  getLength() {
    const length = parseInt(this.readUntil(":"), 10);
    if (Number.isNaN(length)) {
      throw this.error();
    }
    return length;
  }
  getByLength(startSequence, endSequence, callback) {
    const length = this.getLength();
    this.seekExpected(`:${startSequence}`);
    const result = callback(length);
    this.seekExpected(endSequence);
    return result;
  }
};

// src/helpers.ts
function getByteLength(contents) {
  return Buffer.byteLength(contents);
}
__name(getByteLength, "getByteLength");
function isInteger(value) {
  return typeof value === "number" && parseInt(value.toString(), 10) === value;
}
__name(isInteger, "isInteger");

// src/symbol.ts
var ObjectName = Symbol("php.serialize.object.name");

// src/parse.ts
function deserializePairs(parser, length, options) {
  const pairs = [];
  for (let i = 0; i < length; i += 1) {
    const key = deserializeItem(parser, options);
    parser.seekExpected(";");
    const value = deserializeItem(parser, options);
    if (parser.peekAhead(1) === ";") {
      parser.advance(1);
    }
    pairs.push({ key, value });
  }
  return pairs;
}
__name(deserializePairs, "deserializePairs");
function deserializeItem(parser, options) {
  const type = parser.getType();
  if (type === "null") {
    return null;
  }
  if (type === "int" || type === "float") {
    const value = parser.readUntil(";");
    return type === "int" ? parseInt(value, 10) : parseFloat(value);
  }
  if (type === "boolean") {
    const value = parser.readAhead(1);
    return value === "1";
  }
  if (type === "string") {
    return parser.getByLength('"', '"', (length) => parser.readAhead(length));
  }
  if (type === "array-object") {
    const pairs = parser.getByLength("{", "}", (length) => deserializePairs(parser, length, options));
    const isArray = pairs.every((item, idx) => isInteger(item.key) && idx === item.key);
    const result = isArray ? [] : /* @__PURE__ */ Object.create(null);
    pairs.forEach(({ key, value }) => {
      result[key] = value;
    });
    return result;
  }
  if (type === "object") {
    const name = parser.getByLength('"', '"', (length) => parser.readAhead(length));
    parser.seekExpected(":");
    const result = /* @__PURE__ */ Object.create(null);
    result[ObjectName] = name;
    const pairs = parser.getByLength("{", "}", (length) => deserializePairs(parser, length, options));
    pairs.forEach(({ key, value }) => {
      result[key] = value;
    });
    return result;
  }
  if (type === "serializable-class") {
    if (options.strict) {
      throw new TypeError(`can't parse serializable class yet`);
    }
    const name = parser.getByLength('"', '"', (length) => parser.readAhead(length));
    parser.seekExpected(":");
    const result = /* @__PURE__ */ Object.create(null);
    result[ObjectName] = name;
    return parser.getByLength("{", "}", (length) => parser.readAhead(length));
  }
  throw new Error(`Invalid type '${type}' encounter while deserializing`);
}
__name(deserializeItem, "deserializeItem");
function parse(item, givenOptions = {}) {
  const options = {
    strict: givenOptions.strict ?? true,
    encoding: givenOptions.encoding ?? "utf-8"
  };
  const parser = new Parser(Buffer.isBuffer(item) ? item : Buffer.from(item), 0, options);
  return deserializeItem(parser, options);
}
__name(parse, "parse");

// src/stringify.ts
function serializePlainObject(item) {
  const processed = Object.keys(item).map((key) => `${stringify(key)}${stringify(item[key])}`);
  const { length } = processed.filter((entry) => typeof entry !== "undefined");
  return `${length}:{${processed.join("")}}`;
}
__name(serializePlainObject, "serializePlainObject");
function serializeArray(item) {
  const processed = item.map((value, index) => `${stringify(index)}${stringify(value)}`);
  const { length } = processed.filter((entry) => typeof entry !== "undefined");
  return `${length}:{${processed.join("")}}`;
}
__name(serializeArray, "serializeArray");
function stringify(item) {
  if (item === null) {
    return "N;";
  }
  if (item === void 0) {
    return "N;";
  }
  const type = typeof item;
  if (type === "number") {
    if (isInteger(item)) {
      return `i:${item};`;
    }
    return `d:${item.toString().toUpperCase()};`;
  }
  if (type === "bigint") {
    return `i:${item.toString()};`;
  }
  if (type === "string") {
    return `s:${getByteLength(item)}:"${item}";`;
  }
  if (type === "boolean") {
    return `b:${item ? "1" : "0"};`;
  }
  if (Buffer.isBuffer(item)) {
    return `s:${Buffer.byteLength(item)}:"${item.toString()}";`;
  }
  if (Array.isArray(item)) {
    return `a:${serializeArray(item)}`;
  }
  if (item instanceof Map) {
    return `a:${item.size}:{${Array.from(item.entries()).map(([value, key]) => {
      return `${stringify(value)}${stringify(key)}`;
    }).join("")}}`;
  }
  if (type === "symbol") {
    const s = item.toString();
    return `s:${Buffer.byteLength(s)}:"${s}";`;
  }
  if (type !== "object") {
    throw new TypeError(`Unexpected type '${type}' encountered while attempting to serialize`);
  }
  const objectName = item[ObjectName];
  if (objectName) {
    return `O:${Buffer.byteLength(objectName)}:"${objectName}":${serializePlainObject(item)}`;
  }
  return `a:${serializePlainObject(item)}`;
}
__name(stringify, "stringify");
export {
  ObjectName,
  parse,
  stringify
};
//# sourceMappingURL=index.js.map